@@ -141,40 +141,58 @@ static int my_getpass(void *client, char *prompt, char *buffer, int buflen)
 static size_t handle_returned_header (void *ptr, size_t size, size_t nmemb, void *stream)
 {
     auth_client *auth_user = stream;
-    unsigned bytes = size * nmemb;
+    size_t len = size * nmemb;
     client_t *client = auth_user->client;
 
-    if (client)
-    {
+    if (client) {
         auth_t *auth = client->auth;
         auth_url *url = auth->state;
-        if (strncasecmp (ptr, url->auth_header, url->auth_header_len) == 0)
+
+        if (url->auth_header && len >= url->auth_header_len && strncasecmp(ptr, url->auth_header, url->auth_header_len) == 0)
             client->authenticated = 1;
-        if (strncasecmp (ptr, url->timelimit_header, url->timelimit_header_len) == 0)
-        {
+
+        if (url->timelimit_header && len > url->timelimit_header_len && strncasecmp(ptr, url->timelimit_header, url->timelimit_header_len) == 0) {
+            const char *input = ptr;
             unsigned int limit = 0;
-            sscanf ((char *)ptr+url->timelimit_header_len, "%u\r\n", &limit);
-            client->con->discon_time = time(NULL) + limit;
+
+            if (len >= 2 && input[len - 2] == '\r' && input[len - 1] == '\n') {
+                input += url->timelimit_header_len;
+
+                if (sscanf(input, "%u\r\n", &limit) == 1) {
+                    client->con->discon_time = time(NULL) + limit;
+                } else {
+                    ICECAST_LOG_ERROR("Auth backend returned invalid timeline header: Can not parse limit");
+                }
+            } else {
+                ICECAST_LOG_ERROR("Auth backend returned invalid timelimit header.");
+            }
         }
-        if (strncasecmp (ptr, "icecast-auth-message: ", 22) == 0)
-        {
-            char *eol;
-            snprintf (url->errormsg, sizeof (url->errormsg), "%s", (char*)ptr+22);
-            eol = strchr (url->errormsg, '\r');
-            if (eol == NULL)
-                eol = strchr (url->errormsg, '\n');
-            if (eol)
-                *eol = '\0';
+
+        if (len > 24 && strncasecmp(ptr, "icecast-auth-message: ", 22) == 0) {
+            const char *input = ptr;
+            size_t copy_len = len - 24 + 1; /* length of string plus \0-termination */
+
+            if (copy_len > sizeof(url->errormsg)) {
+                copy_len = sizeof(url->errormsg);
+            }
+
+            if (len >= 2 && input[len - 2] == '\r' && input[len - 1] == '\n') {
+                input += 22;
+                memcpy(url->errormsg, input, copy_len);
+                url->errormsg[copy_len-1] = 0;
+            } else {
+                ICECAST_LOG_ERROR("Auth backend returned invalid message header.");
+            }
         }
     }
 
-    return (int)bytes;
+    return len;
 }
 
 /* capture returned data, but don't do anything with it */
 static size_t handle_returned_data (void *ptr, size_t size, size_t nmemb, void *stream)
 {
-    return (int)(size*nmemb);
+    return size * nmemb;
 }
 
 
@@ -191,6 +209,7 @@ static auth_result url_remove_listener (auth_client *auth_user)
     char *userpwd = NULL, post [4096];
     const char *agent;
     char *user_agent, *ipaddr;
+    int ret;
 
     if (url->removeurl == NULL)
         return AUTH_OK;
@@ -223,11 +242,11 @@ static auth_result url_remove_listener (auth_client *auth_user)
     mount = util_url_escape (mountreq);
     ipaddr = util_url_escape (client->con->ip);
 
-    snprintf (post, sizeof (post),
-            "action=listener_remove&server=%s&port=%d&client=%lu&mount=%s"
-            "&user=%s&pass=%s&duration=%lu&ip=%s&agent=%s",
-            server, port, client->con->id, mount, username,
-            password, (long unsigned)duration, ipaddr, user_agent);
+    ret = snprintf(post, sizeof(post),
+                   "action=listener_remove&server=%s&port=%d&client=%lu&mount=%s"
+                   "&user=%s&pass=%s&duration=%lu&ip=%s&agent=%s",
+                   server, port, client->con->id, mount, username,
+                   password, (long unsigned)duration, ipaddr, user_agent);
     free (server);
     free (mount);
     free (username);
@@ -235,6 +254,11 @@ static auth_result url_remove_listener (auth_client *auth_user)
     free (ipaddr);
     free (user_agent);
 
+    if (ret <= 0 || ret >= sizeof(post)) {
+        ICECAST_LOG_ERROR("POST body too long for buffer on mount point \"%H\" client %p.", auth_user->mount, client);
+        return AUTH_FAILED;
+    }
+
     if (strchr (url->removeurl, '@') == NULL)
     {
         if (url->userpwd)
@@ -331,6 +355,11 @@ static auth_result url_add_listener (auth_client *auth_user)
     free (password);
     free (ipaddr);
 
+    if (post_offset <= 0 || post_offset >= sizeof(post)) {
+        ICECAST_LOG_ERROR("POST body too long for buffer on mount point \"%H\" client %p.", auth_user->mount, client);
+        return AUTH_FAILED;
+    }
+
     pass_headers = NULL;
     if (url->pass_headers)
         pass_headers = strdup (url->pass_headers);
@@ -349,15 +378,26 @@ static auth_result url_add_listener (auth_client *auth_user)
             header_val = httpp_getvar (client->parser, cur_header);
             if (header_val)
             {
+                size_t left = sizeof(post) - post_offset;
+                int ret;
                 header_valesc = util_url_escape (header_val);
-                post_offset += snprintf (post+post_offset, sizeof (post)-post_offset, "&%s%s=%s",
+                ret = snprintf (post+post_offset, left, "&%s%s=%s",
                                          url->prefix_headers ? url->prefix_headers : "",
                                          cur_header, header_valesc);
                 free (header_valesc);
+
+                if (ret <= 0 || ret >= left) {
+                    ICECAST_LOG_ERROR("POST body too long for buffer on mount point \"%H\" client %p.", auth_user->mount, client);
+                    free(pass_headers);
+                    return AUTH_FAILED;
+                } else {
+                    post_offset += ret;
+                }
             }
 
 	    cur_header = next_header;
         }
+        free(pass_headers);
     }
 
     if (strchr (url->addurl, '@') == NULL)
@@ -418,6 +458,7 @@ static void url_stream_start (auth_client *auth_user)
     char *stream_start_url;
     int port;
     char post [4096];
+    int ret;
 
     if (url->stream_start == NULL)
     {
@@ -433,11 +474,17 @@ static void url_stream_start (auth_client *auth_user)
     config_release_config ();
     mount = util_url_escape (auth_user->mount);
 
-    snprintf (post, sizeof (post),
-            "action=mount_add&mount=%s&server=%s&port=%d", mount, server, port);
+    ret = snprintf(post, sizeof(post), "action=mount_add&mount=%s&server=%s&port=%d", mount, server, port);
     free (server);
     free (mount);
 
+    if (ret <= 0 || ret >= sizeof(post)) {
+        ICECAST_LOG_ERROR("POST body too long for buffer on mount point \"%H\".", auth_user->mount);
+        auth_release(auth);
+        free(stream_start_url);
+        return;
+    }
+
     if (strchr (url->stream_start, '@') == NULL)
     {
         if (url->userpwd)
@@ -470,6 +517,7 @@ static void url_stream_end (auth_client *auth_user)
     char *stream_end_url;
     int port;
     char post [4096];
+    int ret;
 
     if (url->stream_end == NULL)
     {
@@ -485,11 +533,17 @@ static void url_stream_end (auth_client *auth_user)
     config_release_config ();
     mount = util_url_escape (auth_user->mount);
 
-    snprintf (post, sizeof (post),
-            "action=mount_remove&mount=%s&server=%s&port=%d", mount, server, port);
+    ret = snprintf(post, sizeof(post), "action=mount_remove&mount=%s&server=%s&port=%d", mount, server, port);
     free (server);
     free (mount);
 
+    if (ret <= 0 || ret >= sizeof(post)) {
+        ICECAST_LOG_ERROR("POST body too long for buffer on mount point \"%H\".", auth_user->mount);
+        auth_release(auth);
+        free(stream_end_url);
+        return;
+    }
+
     if (strchr (url->stream_end, '@') == NULL)
     {
         if (url->userpwd)
@@ -519,6 +573,7 @@ static void url_stream_auth (auth_client *auth_user)
     auth_url *url = client->auth->state;
     char *mount, *host, *user, *pass, *ipaddr, *admin="";
     char post [4096];
+    int ret;
 
     if (strchr (url->stream_auth, '@') == NULL)
     {
@@ -553,9 +608,9 @@ static void url_stream_auth (auth_client *auth_user)
         pass = strdup("");
     }
 
-    snprintf (post, sizeof (post),
-            "action=stream_auth&mount=%s&ip=%s&server=%s&port=%d&user=%s&pass=%s%s",
-            mount, ipaddr, host, port, user, pass, admin);
+    ret = snprintf(post, sizeof(post),
+                   "action=stream_auth&mount=%s&ip=%s&server=%s&port=%d&user=%s&pass=%s%s",
+                   mount, ipaddr, host, port, user, pass, admin);
     free (ipaddr);
     free (user);
     free (pass);
@@ -563,6 +618,12 @@ static void url_stream_auth (auth_client *auth_user)
     free (host);
 
     client->authenticated = 0;
+
+    if (ret <= 0 || ret >= sizeof(post)) {
+        ICECAST_LOG_ERROR("POST body too long for buffer on mount point \"%H\" client %p.", auth_user->mount, client);
+        return;
+    }
+
     if (curl_easy_perform (url->handle))
         ICECAST_LOG_WARN("auth to server %s failed with %s", url->stream_auth, url->errormsg);
 }
