@@ -8,9 +8,9 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
- * Copyright 2011,      Thomas B. "dm8tbr" Ruecker <thomas.ruecker@tieto.com>,
- *                      Dave 'justdave' Miller <justdave@mozilla.com>.
- * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
+ * Copyright 2011,      Dave 'justdave' Miller <justdave@mozilla.com>.
+ * Copyright 2011-2014, Thomas B. "dm8tbr" Ruecker <thomas@ruecker.fi>,
+ * Copyright 2011-2014, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,7 +59,7 @@
 #define CONFIG_DEFAULT_GROUP NULL
 #define CONFIG_MASTER_UPDATE_INTERVAL 120
 #define CONFIG_YP_URL_TIMEOUT 10
-#define CONFIG_DEFAULT_CIPHER_LIST "ALL:!aNULL:!ADH:!eNULL:!LOW:!EXP:RC4+RSA:+HIGH:+MEDIUM"
+#define CONFIG_DEFAULT_CIPHER_LIST "ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:kEDH+AESGCM:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES256-SHA:DHE-RSA-AES128-SHA256:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA:DHE-RSA-AES256-SHA:ECDHE-RSA-DES-CBC3-SHA:ECDHE-ECDSA-DES-CBC3-SHA:AES128-GCM-SHA256:AES256-GCM-SHA384:AES128-SHA256:AES256-SHA256:AES128-SHA:AES256-SHA:AES:DES-CBC3-SHA:HIGH:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA"
 
 #ifndef _WIN32
 #define CONFIG_DEFAULT_BASE_DIR "/usr/local/icecast"
@@ -87,6 +87,8 @@ static void _parse_logging(xmlDocPtr doc, xmlNodePtr node, ice_config_t *c);
 static void _parse_security(xmlDocPtr doc, xmlNodePtr node, ice_config_t *c);
 static void _parse_authentication(xmlDocPtr doc, xmlNodePtr node, 
         ice_config_t *c);
+static void _parse_http_headers(xmlDocPtr doc, xmlNodePtr node,
+        ice_config_http_header_t **http_headers);
 static void _parse_relay(xmlDocPtr doc, xmlNodePtr node, ice_config_t *c);
 static void _parse_mount(xmlDocPtr doc, xmlNodePtr node, ice_config_t *c);
 static void _parse_listen_socket(xmlDocPtr doc, xmlNodePtr node, 
@@ -123,6 +125,58 @@ void config_init_configuration(ice_config_t *configuration)
     _set_defaults(configuration);
 }
 
+static void config_clear_http_header(ice_config_http_header_t *header) {
+ ice_config_http_header_t *old;
+
+ while (header) {
+  xmlFree(header->name);
+  xmlFree(header->value);
+  old = header;
+  header = header->next;
+  free(old);
+ }
+}
+
+static inline ice_config_http_header_t * config_copy_http_header(ice_config_http_header_t *header) {
+    ice_config_http_header_t *ret = NULL;
+    ice_config_http_header_t *cur = NULL;
+    ice_config_http_header_t *old = NULL;
+
+    while (header) {
+        if (cur) {
+            cur->next = calloc(1, sizeof(ice_config_http_header_t));
+            old = cur;
+            cur = cur->next;
+        } else {
+            ret = calloc(1, sizeof(ice_config_http_header_t));
+            cur = ret;
+        }
+
+        if (!cur) return ret; /* TODO: do better error handling */
+
+        cur->type   = header->type;
+        cur->name   = (char *)xmlCharStrdup(header->name);
+        cur->value  = (char *)xmlCharStrdup(header->value);
+        cur->status = header->status;
+
+        if (!cur->name || !cur->value) {
+            if (cur->name) xmlFree(cur->name);
+            if (cur->value) xmlFree(cur->value);
+            if (old) {
+                old->next = NULL;
+            } else {
+                ret = NULL;
+            }
+            free(cur);
+            return ret;
+        }
+
+        header = header->next;
+    }
+
+    return ret;
+}
+
 static void config_clear_mount (mount_proxy *mount)
 {
     config_options_t *option;
@@ -155,6 +209,7 @@ static void config_clear_mount (mount_proxy *mount)
         option = nextopt;
     }
     auth_release (mount->auth);
+    config_clear_http_header(mount->http_headers);
     free (mount);
 }
 
@@ -177,7 +232,9 @@ void config_clear(ice_config_t *c)
     relay_server *relay, *nextrelay;
     mount_proxy *mount, *nextmount;
     aliases *alias, *nextalias;
+#ifdef USE_YP
     int i;
+#endif
 
     free(c->config_filename);
 
@@ -263,6 +320,8 @@ void config_clear(ice_config_t *c)
     }
 #endif
 
+    config_clear_http_header(c->http_headers);
+
     memset(c, 0, sizeof(ice_config_t));
 }
 
@@ -413,11 +472,12 @@ static void _parse_root(xmlDocPtr doc, xmlNodePtr node,
         } else if (xmlStrcmp (node->name, XMLSTR("server-id")) == 0) {
             xmlFree (configuration->server_id);
             configuration->server_id = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+            ICECAST_LOG_WARN("Warning, server version string override detected. This may lead to unexpected client software behavior.");
         } else if(xmlStrcmp (node->name, XMLSTR("authentication")) == 0) {
             _parse_authentication(doc, node->xmlChildrenNode, configuration);
         } else if (xmlStrcmp (node->name, XMLSTR("source-password")) == 0) {
             /* TODO: This is the backwards-compatibility location */
-            WARN0("<source-password> defined outside <authentication>. This is deprecated.");
+            ICECAST_LOG_WARN("<source-password> defined outside <authentication>. This is deprecated.");
             if (configuration->source_password) xmlFree(configuration->source_password);
             configuration->source_password = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
         } else if (xmlStrcmp (node->name, XMLSTR("icelogin")) == 0) {
@@ -442,9 +502,13 @@ static void _parse_root(xmlDocPtr doc, xmlNodePtr node,
             _parse_listen_socket(doc, node->xmlChildrenNode, configuration);
         } else if (xmlStrcmp (node->name, XMLSTR("port")) == 0) {
             tmp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
-            configuration->port = atoi(tmp);
-            configuration->listen_sock->port = atoi(tmp);
-            if (tmp) xmlFree(tmp);
+            if (tmp) {
+                configuration->port = atoi(tmp);
+                configuration->listen_sock->port = atoi(tmp);
+                xmlFree(tmp);
+            } else {
+                ICECAST_LOG_WARN("<port> must not be empty.");
+            }
         } else if (xmlStrcmp (node->name, XMLSTR("bind-address")) == 0) {
             if (configuration->listen_sock->bind_address) 
                 xmlFree(configuration->listen_sock->bind_address);
@@ -471,6 +535,8 @@ static void _parse_root(xmlDocPtr doc, xmlNodePtr node,
             configuration->shoutcast_mount = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
         } else if (xmlStrcmp (node->name, XMLSTR("limits")) == 0) {
             _parse_limits(doc, node->xmlChildrenNode, configuration);
+        } else if (xmlStrcmp (node->name, XMLSTR("http-headers")) == 0) {
+            _parse_http_headers(doc, node->xmlChildrenNode, &(configuration->http_headers));
         } else if (xmlStrcmp (node->name, XMLSTR("relay")) == 0) {
             _parse_relay(doc, node->xmlChildrenNode, configuration);
         } else if (xmlStrcmp (node->name, XMLSTR("mount")) == 0) {
@@ -495,6 +561,28 @@ static void _parse_root(xmlDocPtr doc, xmlNodePtr node,
     }
     if (configuration->port == 0)
         configuration->port = 8000;
+
+   /* issue some warnings on bad configurations */
+   if (!configuration->fileserve)
+       ICECAST_LOG_WARN("Warning, serving of static files has been disabled in the config, this will also affect files used by the web interface (stylesheets, images).");
+
+  if (!configuration->hostname || strcmp(configuration->hostname, CONFIG_DEFAULT_HOSTNAME) == 0) {
+      ICECAST_LOG_WARN("Warning, <hostname> not configured, using default value \"%s\". This will cause problems, e.g. with YP directory listings.", CONFIG_DEFAULT_HOSTNAME);
+      if (!configuration->hostname)
+          configuration->hostname = (char *)xmlCharStrdup (CONFIG_DEFAULT_HOSTNAME);
+  }
+
+  if (!configuration->location || strcmp(configuration->location, CONFIG_DEFAULT_LOCATION) == 0) {
+      ICECAST_LOG_WARN("Warning, <location> not configured, using default value \"%s\".", CONFIG_DEFAULT_LOCATION);
+      if (!configuration->location)
+          configuration->location = (char *)xmlCharStrdup (CONFIG_DEFAULT_LOCATION);
+  }
+
+  if (!configuration->admin || strcmp(configuration->admin, CONFIG_DEFAULT_ADMIN) == 0) {
+      ICECAST_LOG_WARN("Warning, <admin> contact not configured, using default value \"%s\".", CONFIG_DEFAULT_ADMIN);
+      if (!configuration->admin)
+          configuration->admin = (char *)xmlCharStrdup (CONFIG_DEFAULT_ADMIN);
+  }
 }
 
 static void _parse_limits(xmlDocPtr doc, xmlNodePtr node, 
@@ -572,7 +660,7 @@ static void _parse_mount(xmlDocPtr doc, xmlNodePtr node,
 	    mount->mounttype = MOUNT_TYPE_DEFAULT;
 	}
 	else {
-	    WARN1("Unknown mountpoint type: %s", tmp);
+	    ICECAST_LOG_WARN("Unknown mountpoint type: %s", tmp);
             config_clear_mount (mount);
             return;
 	}
@@ -707,6 +795,8 @@ static void _parse_mount(xmlDocPtr doc, xmlNodePtr node,
         } else if (xmlStrcmp (node->name, XMLSTR("subtype")) == 0) {
             mount->subtype = (char *)xmlNodeListGetString(
                     doc, node->xmlChildrenNode, 1);
+        } else if (xmlStrcmp (node->name, XMLSTR("http-headers")) == 0) {
+            _parse_http_headers(doc, node->xmlChildrenNode, &(mount->http_headers));
         }
     } while ((node = node->next));
 
@@ -718,10 +808,13 @@ static void _parse_mount(xmlDocPtr doc, xmlNodePtr node,
     }
     else if (mount->mountname != NULL && mount->mounttype == MOUNT_TYPE_DEFAULT)
     {
-    	WARN1("Default mount %s has mount-name set. This is not supported. Behavior may not be consistent.", mount->mountname);
+    	ICECAST_LOG_WARN("Default mount %s has mount-name set. This is not supported. Behavior may not be consistent.", mount->mountname);
     }
-    if (mount->auth)
+    if (mount->auth && mount->mountname) {
         mount->auth->mount = strdup ((char *)mount->mountname);
+    } else if (mount->auth && mount->mounttype == MOUNT_TYPE_DEFAULT ) {
+        mount->auth->mount = strdup ("(default mount)");
+    }
     while(current) {
         last = current;
         current = current->next;
@@ -729,7 +822,7 @@ static void _parse_mount(xmlDocPtr doc, xmlNodePtr node,
 
     if (!mount->fallback_mount && (mount->fallback_when_full || mount->fallback_override))
     {
-        WARN1("Config for mount %s contains fallback options but no fallback mount.", mount->mountname);
+        ICECAST_LOG_WARN("Config for mount %s contains fallback options but no fallback mount.", mount->mountname);
     }
 
     if(last)
@@ -738,6 +831,63 @@ static void _parse_mount(xmlDocPtr doc, xmlNodePtr node,
         configuration->mounts = mount;
 }
 
+static void _parse_http_headers(xmlDocPtr doc, xmlNodePtr node, ice_config_http_header_t **http_headers) {
+    ice_config_http_header_t *header;
+    ice_config_http_header_t *next;
+    char *name = NULL;
+    char *value = NULL;
+    char *tmp;
+    int status;
+    http_header_type type;
+
+    do {
+        if (node == NULL) break;
+        if (xmlIsBlankNode(node)) continue;
+        if (xmlStrcmp (node->name, XMLSTR("header")) != 0) break;
+        if (!(name = (char *)xmlGetProp(node, XMLSTR("name")))) break;
+        if (!(value = (char *)xmlGetProp(node, XMLSTR("value")))) break;
+
+        type = HTTP_HEADER_TYPE_STATIC; /* default */
+        if ((tmp = (char *)xmlGetProp(node, XMLSTR("type")))) {
+            if (strcmp(tmp, "static") == 0) {
+                type = HTTP_HEADER_TYPE_STATIC;
+            } else {
+                ICECAST_LOG_WARN("Unknown type %s for HTTP Header %s", tmp, name);
+                xmlFree(tmp);
+                break;
+            }
+            xmlFree(tmp);
+        }
+
+        status = 0; /* default: any */
+        if ((tmp = (char *)xmlGetProp(node, XMLSTR("status")))) {
+            status = atoi(tmp);
+            xmlFree(tmp);
+        }
+
+        header = calloc(1, sizeof(ice_config_http_header_t));
+        if (!header) break;
+        header->type = type;
+        header->name = name;
+        header->value = value;
+        header->status = status;
+        name = NULL;
+        value = NULL;
+
+        if (!*http_headers) {
+            *http_headers = header;
+            continue;
+        }
+        next = *http_headers;
+        while (next->next) next = next->next;
+        next->next = header;
+    } while ((node = node->next));
+    /* in case we used break we may need to clean those up */
+    if (name)
+	xmlFree(name);
+    if (value)
+	xmlFree(value);
+}
 
 static void _parse_relay(xmlDocPtr doc, xmlNodePtr node,
         ice_config_t *configuration)
@@ -774,8 +924,12 @@ static void _parse_relay(xmlDocPtr doc, xmlNodePtr node,
         }
         else if (xmlStrcmp (node->name, XMLSTR("port")) == 0) {
             tmp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
-            relay->port = atoi(tmp);
-            if(tmp) xmlFree(tmp);
+            if (tmp) {
+                relay->port = atoi(tmp);
+                xmlFree(tmp);
+            } else {
+                ICECAST_LOG_WARN("<port> must not be empty.");
+            }
         }
         else if (xmlStrcmp (node->name, XMLSTR("mount")) == 0) {
             if (relay->mount) xmlFree (relay->mount);
@@ -832,10 +986,14 @@ static void _parse_listen_socket(xmlDocPtr doc, xmlNodePtr node,
 
         if (xmlStrcmp (node->name, XMLSTR("port")) == 0) {
             tmp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
-            if(configuration->port == 0)
-                configuration->port = atoi(tmp);
-            listener->port = atoi(tmp);
-            if(tmp) xmlFree(tmp);
+            if (tmp) {
+                if(configuration->port == 0)
+                    configuration->port = atoi(tmp);
+                listener->port = atoi(tmp);
+                xmlFree(tmp);
+            } else {
+                ICECAST_LOG_WARN("<port> must not be empty.");
+            }
         }
         else if (xmlStrcmp (node->name, XMLSTR("ssl")) == 0) {
             tmp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
@@ -893,7 +1051,7 @@ static void _parse_authentication(xmlDocPtr doc, xmlNodePtr node,
 
         if (xmlStrcmp (node->name, XMLSTR("source-password")) == 0) {
             if (xmlGetProp(node, XMLSTR("mount"))) {
-                ERROR0("Mount level source password defined within global <authentication> section.");
+                ICECAST_LOG_ERROR("Mount level source password defined within global <authentication> section.");
             }
             else {
                 if (configuration->source_password)
@@ -931,7 +1089,7 @@ static void _parse_directory(xmlDocPtr doc, xmlNodePtr node,
     char *tmp;
 
     if (configuration->num_yp_directories >= MAX_YP_DIRECTORIES) {
-        ERROR0("Maximum number of yp directories exceeded!");
+        ICECAST_LOG_ERROR("Maximum number of yp directories exceeded!");
         return;
     }
     do {
@@ -976,8 +1134,12 @@ static void _parse_paths(xmlDocPtr doc, xmlNodePtr node,
             if (configuration->base_dir) xmlFree(configuration->base_dir);
             configuration->base_dir = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
         } else if (xmlStrcmp (node->name, XMLSTR("logdir")) == 0) {
+            if (!(temp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1))) {
+                ICECAST_LOG_WARN("<logdir> must not be empty.");
+                continue;
+            }
             if (configuration->log_dir) xmlFree(configuration->log_dir);
-            configuration->log_dir = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+            configuration->log_dir = temp;
         } else if (xmlStrcmp (node->name, XMLSTR("pidfile")) == 0) {
             if (configuration->pidfile) xmlFree(configuration->pidfile);
             configuration->pidfile = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
@@ -994,14 +1156,22 @@ static void _parse_paths(xmlDocPtr doc, xmlNodePtr node,
             if (configuration->cipher_list) xmlFree(configuration->cipher_list);
             configuration->cipher_list = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
         } else if (xmlStrcmp (node->name, XMLSTR("webroot")) == 0) {
+            if (!(temp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1))) {
+                ICECAST_LOG_WARN("<webroot> must not be empty.");
+                continue;
+            }
             if (configuration->webroot_dir) xmlFree(configuration->webroot_dir);
-            configuration->webroot_dir = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+            configuration->webroot_dir = temp;
             if(configuration->webroot_dir[strlen(configuration->webroot_dir)-1] == '/')
                 configuration->webroot_dir[strlen(configuration->webroot_dir)-1] = 0;
         } else if (xmlStrcmp (node->name, XMLSTR("adminroot")) == 0) {
+            if (!(temp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1))) {
+                ICECAST_LOG_WARN("<adminroot> must not be empty.");
+                continue;
+            }
             if (configuration->adminroot_dir) 
                 xmlFree(configuration->adminroot_dir);
-            configuration->adminroot_dir = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+            configuration->adminroot_dir = (char *)temp;
             if(configuration->adminroot_dir[strlen(configuration->adminroot_dir)-1] == '/')
                 configuration->adminroot_dir[strlen(configuration->adminroot_dir)-1] = 0;
         } else if (xmlStrcmp (node->name, XMLSTR("alias")) == 0) {
@@ -1046,16 +1216,25 @@ static void _parse_paths(xmlDocPtr doc, xmlNodePtr node,
 static void _parse_logging(xmlDocPtr doc, xmlNodePtr node,
         ice_config_t *configuration)
 {
+    char *tmp;
     do {
         if (node == NULL) break;
         if (xmlIsBlankNode(node)) continue;
 
         if (xmlStrcmp (node->name, XMLSTR("accesslog")) == 0) {
+            if (!(tmp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1))) {
+                ICECAST_LOG_WARN("<accesslog> must not be empty.");
+                continue;
+            }
             if (configuration->access_log) xmlFree(configuration->access_log);
-            configuration->access_log = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+            configuration->access_log = tmp;
         } else if (xmlStrcmp (node->name, XMLSTR("errorlog")) == 0) {
+            if (!(tmp = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1))) {
+                ICECAST_LOG_WARN("<errorlog> must not be empty.");
+                continue;
+            }
             if (configuration->error_log) xmlFree(configuration->error_log);
-            configuration->error_log = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+            configuration->error_log = tmp;
         } else if (xmlStrcmp (node->name, XMLSTR("playlistlog")) == 0) {
             if (configuration->playlist_log) xmlFree(configuration->playlist_log);
             configuration->playlist_log = (char *)xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
@@ -1156,6 +1335,9 @@ static void _add_server(xmlDocPtr doc, xmlNodePtr node,
 }
 
 static void merge_mounts(mount_proxy * dst, mount_proxy * src) {
+    ice_config_http_header_t *http_header_next;
+    ice_config_http_header_t **http_header_tail;
+
     if (!dst || !src)
     	return;
 
@@ -1217,6 +1399,15 @@ static void merge_mounts(mount_proxy * dst, mount_proxy * src) {
     	dst->subtype = (char*)xmlStrdup((xmlChar*)src->subtype);
     if (dst->yp_public == -1)
     	dst->yp_public = src->yp_public;
+
+    if (dst->http_headers) {
+        http_header_next = dst->http_headers;
+        while (http_header_next->next) http_header_next = http_header_next->next;
+        http_header_tail = &(http_header_next->next);
+    } else {
+        http_header_tail = &(dst->http_headers);
+    }
+    *http_header_tail = config_copy_http_header(src->http_headers);
 }
 
 static inline void _merge_mounts_all(ice_config_t *c) {
