@@ -3,12 +3,13 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org, 
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org>, 
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
  * Copyright 2011,      Dave 'justdave' Miller <justdave@mozilla.com>.
+ * Copyright 2011-2014, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifndef __CFGFILE_H__
@@ -31,8 +32,27 @@ struct _mount_proxy;
 
 #define XMLSTR(str) ((xmlChar *)(str)) 
 
-typedef struct ice_config_dir_tag
-{
+typedef enum _http_header_type {
+ /* static: headers are passed as is to the client. */
+ HTTP_HEADER_TYPE_STATIC
+} http_header_type;
+
+typedef struct ice_config_http_header_tag {
+    /* type of this header. See http_header_type */
+    http_header_type type;
+
+    /* name and value of the header */
+    char *name;
+    char *value;
+
+    /* filters */
+    int status;
+
+    /* link to the next list element */
+    struct ice_config_http_header_tag *next;
+} ice_config_http_header_t;
+
+typedef struct ice_config_dir_tag {
     char *host;
     int touch_interval;
     struct ice_config_dir_tag *next;
@@ -78,6 +98,8 @@ typedef struct _mount_proxy {
     char *charset;  /* character set if not utf8 */
     int mp3_meta_interval; /* outgoing per-stream metadata interval */
 
+    ice_config_http_header_t *http_headers; /* additional HTTP headers */
+
     char *auth_type; /* Authentication type */
     struct auth_tag *auth;
     char *cluster_password;
@@ -116,8 +138,7 @@ typedef struct _listener_t {
     int ssl;
 } listener_t;
 
-typedef struct ice_config_tag
-{
+typedef struct ice_config_tag {
     char *config_filename;
 
     char *location;
@@ -158,6 +179,8 @@ typedef struct ice_config_tag
     char *master_username;
     char *master_password;
 
+    ice_config_http_header_t *http_headers;
+
     relay_server *relay;
 
     mount_proxy *mounts;
@@ -220,6 +243,3 @@ void config_release_config(void);
 ice_config_t *config_get_config_unlocked(void);
 
 #endif  /* __CFGFILE_H__ */
-
-
-
