@@ -194,7 +194,11 @@ static unsigned long _next_connection_id(void)
 #ifdef HAVE_OPENSSL
 static void get_ssl_certificate (ice_config_t *config)
 {
+#if OPENSSL_VERSION_NUMBER < 0x1000114fL
     SSL_METHOD *method;
+#else
+    const SSL_METHOD *method;
+#endif
     long ssl_opts;
     ssl_ok = 0;
 
@@ -1249,7 +1253,7 @@ static void _handle_shoutcast_compatible (client_queue_t *node)
             client->respcode = 200;
             /* send this non-blocking but if there is only a partial write
              * then leave to header timeout */
-            sock_write (client->con->sock, "OK2\r\nicy-caps:11\r\n\r\n");
+            client_send_bytes(client, "OK2\r\nicy-caps:11\r\n\r\n", 20); /* TODO: Replace Magic Number! */
             node->offset -= (headers - client->refbuf->data);
             memmove (client->refbuf->data, headers, node->offset+1);
             node->shoutcast = 2;
