@@ -9,6 +9,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 /* format_mp3.c
@@ -127,37 +128,39 @@ int format_mp3_get_plugin (source_t *source)
 static void mp3_set_tag (format_plugin_t *plugin, const char *tag, const char *in_value, const char *charset)
 {
     mp3_state *source_mp3 = plugin->_state;
-    unsigned int len;
-    const char meta[] = "StreamTitle='";
-    int size = sizeof (meta) + 1;
-    char *value;
-
-    if (tag==NULL || in_value == NULL)
-        return;
+    char *value = NULL;
 
     /* protect against multiple updaters */
     thread_mutex_lock (&source_mp3->url_lock);
 
-    value = util_conv_string (in_value, charset, plugin->charset);
-    if (value == NULL)
-        value = strdup (in_value);
+    if (tag==NULL)
+    {
+        source_mp3->update_metadata = 1;
+        thread_mutex_unlock (&source_mp3->url_lock);
+        return;
+    }
 
-    len = strlen (value)+1;
-    size += len;
+    if (in_value)
+    {
+        value = util_conv_string (in_value, charset, plugin->charset);
+        if (value == NULL)
+            value = strdup (in_value);
+    }
 
     if (strcmp (tag, "title") == 0 || strcmp (tag, "song") == 0)
     {
         free (source_mp3->url_title);
-        free (source_mp3->url_artist);
-        source_mp3->url_artist = NULL;
         source_mp3->url_title = value;
-        source_mp3->update_metadata = 1;
     }
     else if (strcmp (tag, "artist") == 0)
     {
         free (source_mp3->url_artist);
         source_mp3->url_artist = value;
-        source_mp3->update_metadata = 1;
+    }
+    else if (strcmp (tag, "url") == 0)
+    {
+        free (source_mp3->url);
+        source_mp3->url = value;
     }
     else
         free (value);
@@ -204,12 +207,12 @@ static void format_mp3_apply_settings (client_t *client, format_plugin_t *format
 
     if (mount)
     {
-        if (mount->mp3_meta_interval > 0)
+        if (mount->mp3_meta_interval >= 0)
             source_mp3->interval = mount->mp3_meta_interval;
         if (mount->charset)
             format->charset = strdup (mount->charset);
     }
-    if (source_mp3->interval <= 0)
+    if (source_mp3->interval < 0)
     {
         const char *metadata = httpp_getvar (client->parser, "icy-metaint");
         source_mp3->interval = ICY_METADATA_INTERVAL;
@@ -224,8 +227,8 @@ static void format_mp3_apply_settings (client_t *client, format_plugin_t *format
     if (format->charset == NULL)
         format->charset = strdup ("ISO8859-1");
 
-    DEBUG1 ("sending metadata interval %d", source_mp3->interval);
-    DEBUG1 ("charset %s", format->charset);
+    ICECAST_LOG_DEBUG("sending metadata interval %d", source_mp3->interval);
+    ICECAST_LOG_DEBUG("charset %s", format->charset);
 }
 
 
@@ -234,11 +237,12 @@ static void format_mp3_apply_settings (client_t *client, format_plugin_t *format
  */
 static void mp3_set_title (source_t *source)
 {
-    const char meta[] = "StreamTitle='";
-    int size;
+    const char streamtitle[] = "StreamTitle='";
+    const char streamurl[] = "StreamUrl='";
+    size_t size;
     unsigned char len_byte;
     refbuf_t *p;
-    unsigned int len = sizeof(meta) + 2; /* the StreamTitle, quotes, ; and null */
+    unsigned int len = sizeof(streamtitle) + 2; /* the StreamTitle, quotes, ; and null */
     mp3_state *source_mp3 = source->format->_state;
 
     /* make sure the url data does not disappear from under us */
@@ -251,11 +255,19 @@ static void mp3_set_title (source_t *source)
         len += strlen (source_mp3->url_title);
     if (source_mp3->url_artist && source_mp3->url_title)
         len += 3;
+    if (source_mp3->inline_url)
+    {
+        char *end = strstr (source_mp3->inline_url, "';");
+        if (end)
+            len += end - source_mp3->inline_url+2;
+    }
+    else if (source_mp3->url)
+        len += strlen (source_mp3->url) + strlen (streamurl) + 2;
 #define MAX_META_LEN 255*16
     if (len > MAX_META_LEN)
     {
         thread_mutex_unlock (&source_mp3->url_lock);
-        WARN1 ("Metadata too long at %d chars", len);
+        ICECAST_LOG_WARN("Metadata too long at %d chars", len);
         return;
     }
     /* work out the metadata len byte */
@@ -268,14 +280,29 @@ static void mp3_set_title (source_t *source)
     if (p)
     {
         mp3_state *source_mp3 = source->format->_state;
+        int r;
 
         memset (p->data, '\0', size);
         if (source_mp3->url_artist && source_mp3->url_title)
-            snprintf (p->data, size, "%c%s%s - %s';", len_byte, meta,
+            r = snprintf (p->data, size, "%c%s%s - %s';", len_byte, streamtitle,
                     source_mp3->url_artist, source_mp3->url_title);
         else
-            snprintf (p->data, size, "%c%s%s';", len_byte, meta,
+            r = snprintf (p->data, size, "%c%s%s';", len_byte, streamtitle,
                     source_mp3->url_title);
+        if (r > 0)
+        {
+            if (source_mp3->inline_url)
+            {
+                char *end = strstr (source_mp3->inline_url, "';");
+                ssize_t urllen = size;
+                if (end) urllen = end - source_mp3->inline_url + 2;
+                if ((ssize_t)(size-r) > urllen)
+                    snprintf (p->data+r, size-r, "StreamUrl='%s';", source_mp3->inline_url+11);
+            }
+            else if (source_mp3->url)
+                snprintf (p->data+r, size-r, "StreamUrl='%s';", source_mp3->url);
+        }
+        ICECAST_LOG_DEBUG("shoutcast metadata block setup with %s", p->data+1);
         filter_shoutcast_metadata (source, p->data, size);
 
         refbuf_release (source_mp3->metadata);
@@ -289,7 +316,7 @@ static void mp3_set_title (source_t *source)
  * which is 0 or greater.  Check the client in_metadata value afterwards
  * to see if all metadata has been sent
  */
-static int send_mp3_metadata (client_t *client, refbuf_t *associated)
+static int send_stream_metadata (client_t *client, refbuf_t *associated)
 {
     int ret = 0;
     char *metadata;
@@ -355,7 +382,7 @@ static int format_mp3_write_buf_to_client(client_t *client)
         if (client_mp3->in_metadata)
         {
             refbuf_t *associated = refbuf->associated;
-            ret = send_mp3_metadata (client, associated);
+            ret = send_stream_metadata (client, associated);
 
             if (client_mp3->in_metadata)
                 break;
@@ -367,7 +394,7 @@ static int format_mp3_write_buf_to_client(client_t *client)
             unsigned int remaining = client_mp3->interval -
                 client_mp3->since_meta_block;
 
-            /* sending the metadata block */
+            /* leading up to sending the metadata block */
             if (remaining <= len)
             {
                 /* send any mp3 before the metadata block */
@@ -384,11 +411,10 @@ static int format_mp3_write_buf_to_client(client_t *client)
                         break;
                     written += ret;
                 }
-                ret = send_mp3_metadata (client, refbuf->associated);
+                ret = send_stream_metadata (client, refbuf->associated);
                 if (client_mp3->in_metadata)
                     break;
                 written += ret;
-                /* change buf and len */
                 buf += remaining;
                 len -= remaining;
                 /* limit how much mp3 we send if using small intervals */
@@ -595,17 +621,18 @@ static refbuf_t *mp3_get_filter_meta (source_t *source)
             memcpy (meta->data, source_mp3->build_metadata,
                     source_mp3->build_metadata_len);
 
-            DEBUG1("shoutcast metadata %.4080s", meta->data+1);
+	    ICECAST_LOG_DEBUG("shoutcast metadata %.*s", 4080, meta->data+1);
             if (strncmp (meta->data+1, "StreamTitle=", 12) == 0)
             {
                 filter_shoutcast_metadata (source, source_mp3->build_metadata,
                         source_mp3->build_metadata_len);
                 refbuf_release (source_mp3->metadata);
                 source_mp3->metadata = meta;
+                source_mp3->inline_url = strstr (meta->data+1, "StreamUrl='");
             }
             else
             {
-                ERROR0 ("Incorrect metadata format, ending stream");
+                ICECAST_LOG_ERROR("Incorrect metadata format, ending stream");
                 source->running = 0;
                 refbuf_release (refbuf);
                 refbuf_release (meta);
@@ -633,7 +660,7 @@ static int format_mp3_create_client_data(source_t *source, client_t *client)
 {
     mp3_client_data *client_mp3 = calloc(1,sizeof(mp3_client_data));
     mp3_state *source_mp3 = source->format->_state;
-    const char *metadata; 
+    const char *metadata;
     /* the +-2 is for overwriting the last set of \r\n */
     unsigned remaining = 4096 - client->refbuf->len + 2;
     char *ptr = client->refbuf->data + client->refbuf->len - 2;
@@ -652,10 +679,6 @@ static int format_mp3_create_client_data(source_t *source, client_t *client)
         bytes = snprintf (ptr, remaining, "Content-Length: 221183499\r\n");
         remaining -= bytes;
         ptr += bytes;
-        /* avoid browser caching, reported via forum */
-        bytes = snprintf (ptr, remaining, "Expires: Mon, 26 Jul 1997 05:00:00 GMT\r\n");
-        remaining -= bytes;
-        ptr += bytes; 
     }
 
     client->format_data = client_mp3;
@@ -701,7 +724,7 @@ static void write_mp3_to_file (struct source_tag *source, refbuf_t *refbuf)
         return;
     if (fwrite (refbuf->data, 1, refbuf->len, source->dumpfile) < (size_t)refbuf->len)
     {
-        WARN0 ("Write to dump file failed, disabling");
+        ICECAST_LOG_WARN("Write to dump file failed, disabling");
         fclose (source->dumpfile);
         source->dumpfile = NULL;
     }
