@@ -22,12 +22,16 @@ typedef struct {
     /* These are for inline metadata */
     int inline_metadata_interval;
     int offset;
-    unsigned interval;
+    int interval;
     char *url_artist;
     char *url_title;
+    char *url;
+    char *inline_url;
     int update_metadata;
 
     refbuf_t *metadata;
+    refbuf_t *read_data;
+    int read_count;
     mutex_t url_lock;
 
     unsigned build_metadata_len;
@@ -36,6 +40,5 @@ typedef struct {
 } mp3_state;
 
 int format_mp3_get_plugin(struct source_tag *src);
-void mp3_set_tag (format_plugin_t *plugin, char *tag, char *value);
 
 #endif  /* __FORMAT_MP3_H__ */
