@@ -14,6 +14,7 @@
 #define __FSERVE_H__
 
 #include <stdio.h>
+#include "cfgfile.h"
 
 typedef void (*fserve_callback_t)(client_t *, void *);
 
@@ -34,6 +35,7 @@ int fserve_client_create(client_t *httpclient, const char *path);
 int fserve_add_client (client_t *client, FILE *file);
 void fserve_add_client_callback (client_t *client, fserve_callback_t callback, void *arg);
 char *fserve_content_type (const char *path);
+void fserve_recheck_mime_types (ice_config_t *config);
 
 
 #endif
