@@ -38,7 +38,7 @@ void global_initialize(void)
     global.server_sockets = 0;
     global.relays = NULL;
     global.master_relays = NULL;
-    global.running = 0;
+    global.running = ICECAST_BOOTING;
     global.clients = 0;
     global.sources = 0;
     global.source_tree = avl_tree_new(source_compare_sources, NULL);
