@@ -20,19 +20,19 @@
 
 typedef struct _refbuf_tag
 {
+    unsigned int len;
+    unsigned int _count;
     char *data;
-    unsigned long len;
-    int sync_point;
     struct _refbuf_tag *associated;
     struct _refbuf_tag *next;
+    int sync_point;
 
-    unsigned long _count;
 } refbuf_t;
 
 void refbuf_initialize(void);
 void refbuf_shutdown(void);
 
-refbuf_t *refbuf_new(unsigned long size);
+refbuf_t *refbuf_new(unsigned int size);
 void refbuf_addref(refbuf_t *self);
 void refbuf_release(refbuf_t *self);
 
