@@ -55,11 +55,7 @@ void _sig_ignore(int signo)
 
 void _sig_hup(int signo)
 {
-    ICECAST_LOG_INFO("Caught signal %d, scheduling config re-read...", signo);
-
-    global_lock();
     global . schedule_config_reread = 1;
-    global_unlock();
 
     /* some OSes require us to reattach the signal handler */
     signal(SIGHUP, _sig_hup);
@@ -67,8 +63,6 @@ void _sig_hup(int signo)
 
 void _sig_die(int signo)
 {
-    ICECAST_LOG_INFO("Caught signal %d, shutting down...", signo);
-
     /* inform the server to start shutting down */
     global.running = ICECAST_HALTING;
 }
