@@ -8,6 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2012-2014, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifdef HAVE_CONFIG_H
@@ -160,7 +161,7 @@ void stats_shutdown(void)
         n = _stats_threads;
         thread_mutex_unlock(&_stats_mutex);
     } while (n > 0);
-    INFO0("stats thread finished");
+    ICECAST_LOG_INFO("stats thread finished");
 
     /* free the queues */
 
@@ -205,7 +206,7 @@ void stats_event(const char *source, const char *name, const char *value)
 
     if (value && xmlCheckUTF8 ((unsigned char *)value) == 0)
     {
-        WARN2 ("seen non-UTF8 data, probably incorrect metadata (%s, %s)", name, value);
+        ICECAST_LOG_WARN("seen non-UTF8 data, probably incorrect metadata (%s, %s)", name, value);
         return;
     }
     event = build_event (source, name, value);
@@ -234,7 +235,7 @@ void stats_event_conv(const char *mount, const char *name, const char *value, co
             xmlCharEncCloseFunc (handle);
         }
         else
-            WARN1 ("No charset found for \"%s\"", charset);
+            ICECAST_LOG_WARN("No charset found for \"%s\"", charset);
     }
 
     stats_event (mount, name, metadata);
@@ -268,12 +269,12 @@ void stats_event_args(const char *source, char *name, char *format, ...)
     if (name == NULL)
         return;
     va_start(val, format);
-    ret = vsnprintf(buf, 1024, format, val);
+    ret = vsnprintf(buf, sizeof(buf), format, val);
     va_end(val);
 
     if (ret < 0 || (unsigned int)ret >= sizeof (buf))
     {
-        WARN2 ("problem with formatting %s stat %s",
+        ICECAST_LOG_WARN("problem with formatting %s stat %s",
                 source==NULL ? "global" : source, name);
         return;
     }
@@ -313,7 +314,7 @@ char *stats_get_value(const char *source, const char *name)
 void stats_event_inc(const char *source, const char *name)
 {
     stats_event_t *event = build_event (source, name, NULL);
-    /* DEBUG2("%s on %s", name, source==NULL?"global":source); */
+    /* ICECAST_LOG_DEBUG("%s on %s", name, source==NULL?"global":source); */
     if (event)
     {
         event->action = STATS_EVENT_INC;
@@ -324,7 +325,7 @@ void stats_event_inc(const char *source, const char *name)
 void stats_event_add(const char *source, const char *name, unsigned long value)
 {
     stats_event_t *event = build_event (source, name, NULL);
-    /* DEBUG2("%s on %s", name, source==NULL?"global":source); */
+    /* ICECAST_LOG_DEBUG("%s on %s", name, source==NULL?"global":source); */
     if (event)
     {
         event->value = malloc (16);
@@ -349,7 +350,7 @@ void stats_event_sub(const char *source, const char *name, unsigned long value)
 /* decrease the value in the provided stat by 1 */
 void stats_event_dec(const char *source, const char *name)
 {
-    /* DEBUG2("%s on %s", name, source==NULL?"global":source); */
+    /* ICECAST_LOG_DEBUG("%s on %s", name, source==NULL?"global":source); */
     stats_event_t *event = build_event (source, name, NULL);
     if (event)
     {
@@ -463,7 +464,7 @@ static void modify_node_event (stats_node_t *node, stats_event_t *event)
                 value = atoll (node->value) - atoll (event->value);
                 break;
             default:
-                WARN2 ("unhandled event (%d) for %s", event->action, event->source);
+                ICECAST_LOG_WARN("unhandled event (%d) for %s", event->action, event->source);
                 break;
         }
         str = malloc (16);
@@ -476,9 +477,9 @@ static void modify_node_event (stats_node_t *node, stats_event_t *event)
     free (node->value);
     node->value = str;
     if (event->source)
-        DEBUG3 ("update \"%s\" %s (%s)", event->source, node->name, node->value);
+        ICECAST_LOG_DEBUG("update \"%s\" %s (%s)", event->source, node->name, node->value);
     else
-        DEBUG2 ("update global %s (%s)", node->name, node->value);
+        ICECAST_LOG_DEBUG("update global %s (%s)", node->name, node->value);
 }
 
 
@@ -486,7 +487,7 @@ static void process_global_event (stats_event_t *event)
 {
     stats_node_t *node;
 
-    /* DEBUG3("global event %s %s %d", event->name, event->value, event->action); */
+    /* ICECAST_LOG_DEBUG("global event %s %s %d", event->name, event->value, event->action); */
     if (event->action == STATS_EVENT_REMOVE)
     {
         /* we're deleting */
@@ -522,7 +523,7 @@ static void process_source_event (stats_event_t *event)
         snode = (stats_source_t *)calloc(1,sizeof(stats_source_t));
         if (snode == NULL)
             return;
-        DEBUG1 ("new source stat %s", event->source);
+        ICECAST_LOG_DEBUG("new source stat %s", event->source);
         snode->source = (char *)strdup(event->source);
         snode->stats_tree = avl_tree_new(_compare_stats, NULL);
         if (event->action == STATS_EVENT_HIDDEN)
@@ -542,7 +543,7 @@ static void process_source_event (stats_event_t *event)
             /* adding node */
             if (event->value)
             {
-                DEBUG2 ("new node %s (%s)", event->name, event->value);
+                ICECAST_LOG_DEBUG("new node %s (%s)", event->name, event->value);
                 node = (stats_node_t *)calloc(1,sizeof(stats_node_t));
                 node->name = (char *)strdup(event->name);
                 node->value = (char *)strdup(event->value);
@@ -554,7 +555,7 @@ static void process_source_event (stats_event_t *event)
         }
         if (event->action == STATS_EVENT_REMOVE)
         {
-            DEBUG1 ("delete node %s", event->name);
+            ICECAST_LOG_DEBUG("delete node %s", event->name);
             avl_delete(snode->stats_tree, (void *)node, _free_stats);
             return;
         }
@@ -579,32 +580,74 @@ static void process_source_event (stats_event_t *event)
     }
     if (event->action == STATS_EVENT_REMOVE)
     {
-        DEBUG1 ("delete source node %s", event->source);
+        ICECAST_LOG_DEBUG("delete source node %s", event->source);
         avl_delete(_stats.source_tree, (void *)snode, _free_source_stats);
     }
 }
 
+/* NOTE: implicit %z is added to format string. */
+static inline void __format_time(char * buffer, size_t len, const char * format) {
+    time_t now = time(NULL);
+    struct tm local;
+    char tzbuffer[32];
+    char timebuffer[128];
+#ifdef _WIN32
+    struct tm *thetime;
+    int time_days, time_hours, time_tz;
+    int tempnum1, tempnum2;
+    char sign;
+#endif
+
+    localtime_r (&now, &local);
+#ifndef _WIN32
+    strftime (tzbuffer, sizeof(tzbuffer), "%z", &local);
+#else
+    thetime = gmtime (&now);
+    time_days = local.tm_yday - thetime->tm_yday;
+
+    if (time_days < -1) {
+        tempnum1 = 24;
+    } else {
+        tempnum1 = 1;
+    }
+
+    if (tempnum1 < time_days) {
+        tempnum2 = -24;
+    } else {
+        tempnum2 = time_days*24;
+    }
+
+    time_hours = (tempnum2 + local.tm_hour - thetime->tm_hour);
+    time_tz = time_hours * 60 + local.tm_min - thetime->tm_min;
+
+    if (time_tz < 0) {
+        sign = '-';
+        time_tz = -time_tz;
+    } else {
+        sign = '+';
+    }
+
+    snprintf(tzbuffer, sizeof(tzbuffer), "%c%.2d%.2d", sign, time_tz / 60, time_tz % 60);
+#endif
+    strftime (timebuffer, sizeof(timebuffer), format, &local);
+
+    snprintf(buffer, len, "%s%s", timebuffer, tzbuffer);
+}
 
 void stats_event_time (const char *mount, const char *name)
 {
-    time_t now = time(NULL);
-    struct tm local;
     char buffer[100];
 
-    localtime_r (&now, &local);
-    strftime (buffer, sizeof (buffer), "%a, %d %b %Y %H:%M:%S %z", &local);
+    __format_time(buffer, sizeof(buffer), "%a, %d %b %Y %H:%M:%S ");
     stats_event (mount, name, buffer);
 }
 
 
 void stats_event_time_iso8601 (const char *mount, const char *name)
 {
-    time_t now = time(NULL);
-    struct tm local;
     char buffer[100];
 
-    localtime_r (&now, &local);
-    strftime (buffer, sizeof (buffer), "%FT%T%z", &local);
+    __format_time(buffer, sizeof(buffer), "%Y-%m-%dT%H:%M:%S");
     stats_event (mount, name, buffer);
 }
 
@@ -642,7 +685,7 @@ static void *_stats_thread(void *arg)
     stats_event (NULL, "stats_connections", "0");
     stats_event (NULL, "listener_connections", "0");
 
-    INFO0 ("stats thread started");
+    ICECAST_LOG_INFO("stats thread started");
     while (_stats_running) {
         thread_mutex_lock(&_global_event_mutex);
         if (_global_event_queue.head != NULL) {
@@ -865,7 +908,7 @@ void *stats_connection(void *arg)
     stats_event_t *event;
     event_listener_t listener;
 
-    INFO0 ("stats client starting");
+    ICECAST_LOG_INFO("stats client starting");
 
     event_queue_init (&listener.queue);
     /* increment the thread count */
@@ -901,7 +944,7 @@ void *stats_connection(void *arg)
 
     thread_mutex_destroy (&listener.mutex);
     client_destroy (client);
-    INFO0 ("stats client finished");
+    ICECAST_LOG_INFO("stats client finished");
 
     return NULL;
 }
@@ -1062,7 +1105,7 @@ void stats_clear_virtual_mounts (void)
         {
             /* no source_t is reserved so remove them now */
             snode = avl_get_next (snode);
-            DEBUG1 ("releasing %s stats", src->source);
+            ICECAST_LOG_DEBUG("releasing %s stats", src->source);
             avl_delete (_stats.source_tree, src, _free_source_stats);
             continue;
         }
